# -*- coding: utf-8 -*-

"""
@author: python273
@contact: https://python273.pw
@license MIT License, see LICENSE file

Copyright (C) 2018
"""

__author__ = 'python273'
__version__ = '1.4.1'

from .api import Telegraph, TelegraphException
from .upload import upload_file